pub fn clamp_add(a: i64, b: i64) -> i64Expand description
Performs saturating addition on i64.
This implements a clamping policy in the case of overflow, returning
i64::MIN or i64::MAX.
pub fn clamp_add(a: i64, b: i64) -> i64Performs saturating addition on i64.
This implements a clamping policy in the case of overflow, returning
i64::MIN or i64::MAX.